# Copyright (c) 2020 Cloudera, Inc.
#
# agent.py: Script used to provide health check for FreeIPA.
import importlib.util
import logging
import logging.config

from ipahealthagent.config import Config
from ipahealthagent.main import bp as main_bp
from ipahealthagent.main.utils import get_plugins
from ipahealthagent.main.utils import loaded_plugins
from flask import Flask

LOG_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "simple": {
            "format": "%(asctime)s [%(threadName)s-%(thread)d] - %(name)s.%(funcName)s(%(lineno)d) "
                      "- %(levelname)6s - %(message)s"
        }
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "level": "INFO",
            "formatter": "simple",
            "stream": "ext://sys.stdout"
        },
        "file": {
            "class": "logging.handlers.RotatingFileHandler",
            "formatter": "simple",
            "filename": "./ipahealthagent.log",
            "maxBytes": 256000,
            "backupCount": 1,
            "level": "DEBUG",
        }
    },
    "loggers": {
        "simpleExample": {
            "level": "DEBUG",
            "handlers": [
                "console"
            ],
            "propagate": "yes"
        }
    },
    "root": {
        "level": "DEBUG",
        "handlers": [
            "console",
            "file"
        ]
    }
}

LOG = logging.getLogger('freeipaagent')


def config_logger():
    logging.config.dictConfig(LOG_CONFIG)


def create_app(config_class=Config):
    config_logger()
    app = Flask(__name__)
    app.config.from_object(config_class)

    app.register_blueprint(main_bp)

    plugins = get_plugins()

    for plugin in plugins:
        try:
            LOG.info('Load: ' + plugin['name'])
            module_spec = plugin['metadata']
            mod_from_spec = importlib.util.module_from_spec(plugin['metadata'])
            module_spec.loader.exec_module(mod_from_spec)
            loaded_plugins.append(mod_from_spec)
        except Exception as e:
            LOG.error('Unable to load module ' + plugin['name'] + ' ' + str(e), exc_info=True)

    LOG.info('Loaded plugins: ' + str(loaded_plugins))
    return app


if __name__ == '__main__':
    create_app().run(host='0.0.0.0', port=Config.PORT)
