# Copyright (c) 2020 Cloudera, Inc.
#
import json
from flask import Response
from timeit import default_timer as timer

from ipahealthagent import logging
from ipahealthagent.logfilter import DuplicateFilter
from ipahealthagent.main import bp
from ipahealthagent.main.result import (Status, CheckResult, ClusterCheckResult,
                                        PluginCheckResult, )
from ipahealthagent.main.utils import get_loaded_plugins
from ipahealthagent.main.cluster import is_clustered

LOG = logging.getLogger(__name__)
LOG.addFilter(DuplicateFilter())


def merge_status(status1, status2):
    result = Status.HEALTHY
    if status1 == Status.DEGRADED or status2 == Status.DEGRADED:
        result = Status.DEGRADED
    if status1 == Status.UNHEALTHY or status2 == Status.UNHEALTHY:
        result = Status.UNHEALTHY
    if status1 == Status.ERROR or status2 == Status.ERROR:
        result = Status.ERROR
    return result


def consolidate_health(result):
    status_code = 200
    for onePluinStat in result.plugin_stat:
        result.status = merge_status(result.status, onePluinStat.status)
    if isinstance(result, CheckResult):
        for oneCheck in result.checks:
            result.status = merge_status(result.status, oneCheck.status)
    else:
        for replica in result.replicas:
            for oneCheck in replica.checks:
                replica.status = merge_status(replica.status, oneCheck.status)
            result.status = merge_status(result.status, replica.status)
    if result.status == Status.UNHEALTHY:
        status_code = 503
    return status_code


def run_check(plugin, in_cluster, parent_checks, parent_plugin):
    plugin_result = PluginCheckResult(plugin.name)
    start = timer()
    try:
        check_results = plugin.run(in_cluster)
        plugin_result.response_time = int(round((timer() - start) * 1000))
        parent_checks.extend(check_results)
        parent_plugin.append(plugin_result)
    except Exception as e:
        LOG.error('Failed run for plugin: %s', plugin, exc_info=True)
        plugin_result.response_time = int(round((timer() - start) * 1000))
        plugin_result.status = Status.ERROR
        plugin_result.detail.update({"Exception": str(e)})
        parent_plugin.append(plugin_result)


def log_result(status_code, result):
    if status_code == 200:
        LOG.info("FreeIPA is healthy")
    else:
        LOG.warning("FreeIPA is unhealthy: %s", result)


@bp.route('/', methods=['GET'])
def node_check():
    result = CheckResult()
    in_cluster = is_clustered()
    for plugin in get_loaded_plugins():
        if not hasattr(plugin,
                       'plugin_type') or (plugin.plugin_type != 'CLUSTER_HEALTH' and plugin.plugin_type != 'HEALTH_CHECK'):
            run_check(plugin, in_cluster, result.checks,
                      result.plugin_stat)

    status_code = consolidate_health(result)
    log_result(status_code, result)
    return Response(str(json.dumps(result, sort_keys=True)),
                    status=status_code, mimetype='application/json')


@bp.route('/cluster', methods=['GET'])
def cluster_check():
    result = ClusterCheckResult()
    in_cluster = is_clustered()
    for plugin in get_loaded_plugins():
        if hasattr(plugin,
                   'plugin_type') and plugin.plugin_type == 'CLUSTER_HEALTH':
            run_check(plugin, in_cluster, result.replicas,
                      result.plugin_stat)

    status_code = consolidate_health(result)
    log_result(status_code, result)
    return Response(str(json.dumps(result, sort_keys=True)),
                    status=status_code, mimetype='application/json')

@bp.route('/ping', methods=['GET'])
def ping():
    return Response('pong', 200, mimetype='text/plain')

@bp.route('/lb-health-check', methods=['GET'])
def lb_health_check():
    result = CheckResult()
    for plugin in get_loaded_plugins():
        if hasattr(plugin,
                   'plugin_type') and plugin.plugin_type == 'HEALTH_CHECK':
            run_check(plugin, False, result.checks,
                      result.plugin_stat)

    status_code = consolidate_health(result)
    log_result(status_code, result)
    return Response(str(json.dumps(result, sort_keys=True)),
                    status=status_code, mimetype='application/json')
