# Copyright (c) 2020 Cloudera, Inc.
#
# __init__.py: Plugin to check cloudwatch alarms

import json
import os
import requests
import subprocess
from subprocess import CalledProcessError

from ipahealthagent import logging
from ipahealthagent.main.result import Status, CheckEntry
from ipahealthagent.main.utils import check_config
from ipahealthagent.plugins.cloudwatch_check import config

name = 'CLOUDWATCH_CHECK'
LOG = logging.getLogger(name)
UUID_FILE = '/sys/devices/virtual/dmi/id/product_uuid'
INSTANCE_META_URL = 'http://169.254.169.254/latest/dynamic/instance-identity' \
                    '/document'
config_mtime = None


def run(is_part_of_cluster=False):
    global config_mtime
    config_mtime = check_config(config_mtime, config)
    checks = []
    if config.Config.enabled and os.path.exists(UUID_FILE):
        try:
            # First check the standard file if it's an EC2 instance
            with open(UUID_FILE, 'r') as uuid_file:
                line = uuid_file.readline()
                if line.startswith('EC2'):
                    # Double check the metadata since small chance random
                    # uuid can start with EC2
                    # If it's not an ec2 instance, don't add to results.
                    metadata = requests.get(INSTANCE_META_URL, verify=False,
                                            timeout=0.5).json()
                    if ('instanceId' in metadata and len(
                        metadata['instanceId']) > 0 and
                        'region' in metadata and len(metadata['region']) > 0):
                        check = CheckEntry(name)
                        checks.append(check)
                        check.check_id = "cloudwatch"
                        try:
                            cw_output = subprocess.check_output(
                                '/bin/aws cloudwatch describe-alarms'
                                ' --region ' + metadata['region'] +
                                ' --alarm-names ' + metadata['instanceId'] +
                                '-Status-Check-Failed-System', shell=True,
                                stderr=subprocess.STDOUT)
                            cw_json = json.loads(cw_output)
                            if len(cw_json['MetricAlarms']) == 0:
                                LOG.error("No alarms defined.")
                                check.detail.update(
                                    {'Status': 'No alarms defined.'})
                                check.status = Status.ERROR
                            elif 'OK' in cw_json['MetricAlarms'][0][
                                'StateValue']:
                                check.detail.update({'Status': cw_json[
                                    'MetricAlarms'][0]['StateValue']})
                            else:
                                check.status = Status.ERROR
                                check.detail.update({'Status': cw_output})

                        except CalledProcessError as e:
                            LOG.error('CalledProcessError', exc_info=True)
                            LOG.error(str(e.output))
                            check.status = Status.ERROR
                            check.detail.update({'Status': str(e.output)})

        except Exception as e:
            # Don't return results on exceptions outside the inner block.
            LOG.warning('Failed cloudwatch check', exc_info=True)

    return checks
