# Copyright (c) 2020 Cloudera, Inc.
#
# __init__.py: Plugin to check systemctl services

import dbus
import threading
import glob

from ipahealthagent import logging
from ipahealthagent.main.result import Status, CheckEntry
from ipahealthagent.main.utils import check_config
from ipahealthagent.plugins.service_check import config

name = 'SERVICE_CHECK'
LOG = logging.getLogger(name)

config_mtime = None

dbus_mutex = threading.Lock()

class SystemBus(object):
    def __enter__(self):
        global dbus_mutex
        dbus_mutex.acquire()
        self.bus = dbus.SystemBus()
        return self

    def __exit__(self, type, value, traceback):
        global dbus_mutex
        dbus_mutex.release()
        self.bus.close()

    def get_object(self, bus_name, object_path):
        return self.bus.get_object(bus_name, object_path)


def newResult(service_name, status, expected, value):
    check = CheckEntry(name)
    check.check_id = service_name + '_' + status
    check.detail.update({status: value})
    if expected != value:
        check.status = Status.UNHEALTHY
    return check

def run(is_part_of_cluster=False):
    global config_mtime
    config_mtime = check_config(config_mtime, config)
    last_exception = Exception("Unknown")

    service_list = config.Config.services[:]
    if is_part_of_cluster:
        service_list += config.Config.cluster_services

    dirsrv_instance = glob.glob('/etc/sysconfig/dirsrv-*')[0].split('/')[3]
    max_attempts = config.Config.max_retries + 1
    for attempt in range(max_attempts):
        try:
            checks = []
            with SystemBus() as bus:
                for service_name in service_list:
                    if service_name == 'dirsrv':
                        service_name = service_name + '@' + dirsrv_instance.split('-', 1)[1]
                    service_name += '.service'
                    systemd = bus.get_object('org.freedesktop.systemd1',
                                             '/org/freedesktop/systemd1')
                    manager = dbus.Interface(systemd, 'org.freedesktop.systemd1.Manager')
                    try:
                        service = bus.get_object('org.freedesktop.systemd1',
                                                 object_path=manager.GetUnit(service_name))
                        interface = dbus.Interface(service,
                                                   dbus_interface='org.freedesktop.DBus.Properties')
                        loaded = interface.Get('org.freedesktop.systemd1.Unit', 'LoadState')
                        checks.append(newResult(service_name, "loaded", "loaded", loaded))

                        active = interface.Get('org.freedesktop.systemd1.Unit',
                                               'ActiveState')
                        checks.append(newResult(service_name, "active", "active", active))

                        sub = interface.Get('org.freedesktop.systemd1.Unit', 'SubState')
                        checks.append(newResult(service_name, "sub", "running", sub))

                    except Exception as e:
                        LOG.error("bad service " + service_name + ":" + str(e), exc_info=True)
                        check = CheckEntry(name)
                        checks.append(check)
                        check.check_id = service_name
                        check.status = Status.UNHEALTHY
                        check.detail.update({"Exception": str(e)})
            break
        except Exception as e:
            LOG.error("failed to get service status from dbus (" + str(attempt + 1) + "/" + str(max_attempts) +"): ", exc_info=True)
            last_exception = e
            continue
    else:
        LOG.error(name + " failed to run, returning with exception: " + str(last_exception))
        raise last_exception

    return checks
